% Chapter_1_data_fit.m
%
% Computational Optimization: Success in Practice
% Chapter 1: Introduction to Optimization
% Example 1.2: Data Fitting
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc;

% input data
data = [2 1; 4 9; 7 6];
solution = 1;

% Getting solution according to the solution code: 1, 2 or 3
if solution==1
  % Solution 1: using separate m-code in a separate file DataFitM.m
  DataFitM;

elseif solution==2
  % Solution 2: using user-defined function placed at the end of this code
  a = DataFitFn(data);

elseif solution==3
  % Solution 3: using Matlab's polyfit function P = POLYFIT(X,Y,N)
  % run 'help polyfit' in command line to get polyfit function description
  a = polyfit(data(:,1),data(:,2),size(data,1)-1);
  a = a(end:-1:1)'; % getting the same format of vector a
  
else
  disp('error: solution code is unknown!');
  return;
end

% Dispaying solution
disp('Optimal solution:');
disp(a');

% Polynomial preparation (for visualization)
xx = min(data(:,1)):0.1:max(data(:,1)); % discretizing x
polynom = a(1)*ones(1,length(xx)) + a(2)*xx + a(3)*xx.^2; % row-vector form

% Visualization
figure(1);
plot(xx,polynom,'-r', 'LineWidth',2.5); hold on; % plotting polynomial curve
plot(data(:,1),data(:,2),'.b','MarkerSize',30); % plotting data points
title(['a_1 = ' num2str(a(1)) ', a_2 = ' num2str(a(2)) ', a_3 = ' num2str(a(3))]); % setting title
legend('polynomial curve','data points','Location','SouthEast'); legend boxoff; % setting legend
set(gca,'XLim',[xx(1)-1 xx(end)+1],'FontSize',14); box off; % graph settings

return % end of the main code

% Solution 2: using user-defined function DataFitFn
function a = DataFitFn(data)

  x = data(:,1);
  y = data(:,2);
  A = [ones(length(x),1) x x.^2];
  a = A^-1 * y;

end