% Chapter_10_opt_lotka_volterra_enhanced.m
%
% Computational Optimization: Success in Practice
% Chapter 10: Implementing Regularization Techniques
% Section 10.3: Examples of Numerical Implementation
%
% Optimization framework for the Lotka-Volterra (LV) predator-prey model:
%    * Matlab ODE solvers ode23 & ode45 are used to solve the forward
%      and adjoint problems
%    * control variable u(t) is set as exact function used for generating
%      synthetic measurements
%    * measurements x1*(t) & x2*(t) are continuous functions
%
% Included:
% - modes:   regular OPT & TEST (for gradient-based approaches)
% - methods (gradient-based):
%          * steepest descent (SD)
%          * quasi-Newton (BFGS)
%          * Conjugate Gradient (CG)
% - stepsize methods:
%          * constant (CONST)
%          * Golden Section (GS) Search
%          * bracketing-Brent toolbox (BB)
% - regularization:
%          * preconditioning
%          * simple projection for control boundness
%          * state- and control-space regularization to improve endpoint solution
% - output: functional value (in file)
%           iterative solutions (in file)
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear; tic;

% setting INPUT parameters
params;

% paths for supplemental MATLAB codes
addpath algorithms;                % algorithms for main code
addpath functions;                 % m-functions

% INItialization
initialize;

% creating synthetic measurements
measurements;

% choosing mode
if strcmp(mode,'OPT')
  mode_OPT;
elseif strcmp(mode,'TEST')
  mode_TEST;
else
  disp(['error: Unknown mode ' mode ' is chosen!']); return;
end

% final output
if (exCode==0)
  return;
elseif strcmp(mode,'OPT')
  file_output;
  fprintf(['Terminated in ' num2str(k) ' iteration(s): termination condition #' num2str(exCode) ' satisfied!\n']);
end
fprintf(['We are fully done! CPU elapsed time = ' num2str(toc) 's\n\n']);
