% fn_precond_bvp4c.m
%
% updating gradient with preconditioning by solving ODE-2
% using MATLAB's function bvp4c:
%
% y - ell^2 y'' = f, where f = grad_L2, y = grad_H1
% y'(a) = y'(b) = 0
%
% converted to a system of ODE-1:
% 
% y(1)' = y(2)
% y(2)' = (y(1)-f)/ell^2
%

function grad_precond = fn_precond_bvp4c(grad_old,tt,ell)
  % initial guess for solution
  solinit = bvpinit(tt,[mean(grad_old),0]);
  % solving BVP-2 using bvp4c
  sol = bvp4c(@(t,y) deriv(t,y,ell,grad_old,tt),@bcs,solinit);
  yint = deval(sol,tt);
  grad_precond = (yint(1,:))';
return

% deriving RHS of ODE system
function dydx = deriv(t,y,ell,grad_old,tt)
  dydx = [y(2); (y(1)-interp1(tt,grad_old,t))/ell^2];
 
% boundary conditions y'(a)=0, y'(b)=0
function res = bcs(ya,yb)
  res = [ya(2)-0; yb(2)-0];