% fn_precond_fd.m
%
% updating gradient with preconditioning by solving ODE-2 using FD
%

function grad_precond = fn_precond_fd(grad_old,ht,ell)
  Nt = length(grad_old) - 2; % # of grid points  
  E = ones(Nt,1);
  
  % matrix P for preconditioning operator
  P = spdiags([(-ell^2/(ht^2))*E (1.0+2.0*ell^2/(ht^2))*E (-ell^2/(ht^2))*E], -1:1, Nt, Nt);
  % boundary conditions
  P(1,1) = 1 + 2*ell^2/(3*ht^2);
  P(1,2) = -2*ell^2/(3*ht^2);
  P(Nt,Nt-1) = -2*ell^2/(3*ht^2);
  P(Nt,Nt) = 1 + 2*ell^2/(3*ht^2);
  
  % gradient update
  grad_new = P\grad_old(2:Nt+1);
  grad_precond = [(4*grad_new(1)-grad_new(2))/3; grad_new; (4*grad_new(Nt)-grad_new(Nt-1))/3];
return