% direction_search.m
%
% Algorithm for finding search direction using:
% - Steepest Descent (SD)
% - Conjugate Gradient (CG)
% - quasi-Newton (BFGS)
%

switch method
    
  case 'SD'
    d = -grad(a,params);

  case 'CG'
    grad_curr = grad(a,params);
    if (k==0)
      d = -grad_curr;
    else
      d = -grad_curr + (grad_curr'*grad_curr)/(grad_old'*grad_old)*d;
    end
    grad_old = grad_curr;

  case 'BFGS'
    if (rem(k,BFGSrst)==0)
      D = eye(length(a),length(a));
      grad_old = grad(a,params);
    else
      grad_curr = grad(a,params);
      s = a - a_old;
      y = grad_curr - grad_old;
      grad_old = grad_curr;
      r = 1/(y'*s);
      D = (eye(length(a),length(a))-r*s*y')*D*(eye(length(a),length(a))-r*y*s')+r*s*s';
    end
    d = -D*grad(a,params);
    
end