% kappa_test.m
%
% Kappa-test for Chapter_12_opt_pde.m
%

% Kappa-test #1: cheap test

% L2 inner product (in denominator) by gradient
paramsLine = ['-p1 ' num2str(numCtrls)];
[status, cmdout] = system(['FreeFem++ freefem/kappa_grad.edp ' paramsLine]);
fn_stat(status, 0, cmdout, ffDebug);
G = load('temp/opt_kappa.dat');

for k = 1:16
  eps(k) = 10^(k-16);
  kappaC(k) = (f(eps(k),a,a,params)-obj)/(eps(k)*G);
end

% visualization for cheap kappa-test #1
figure(1);
semilogx(eps,kappaC,'r.-','MarkerSize',MarkerSizeSmall,'LineWidth', LineWidth); hold on;
plot([eps(1) eps(end)], [1 1], '--k'); axis([eps(1) eps(end) 0.95 1.05]);
xlabel('\epsilon','FontSize',FontSize); ylabel('\kappa(\epsilon)','FontSize',FontSize);
title('kappa-test (cheap)','FontSize',FontSize);
set(gca,'FontSize',FontSize-2); box off;

figure(2);
loglog(eps,abs(kappaC-1),'r.-','MarkerSize',MarkerSizeSmall,'LineWidth', LineWidth); hold on;
xlabel('\epsilon','FontSize',FontSize); ylabel('| \kappa(\epsilon) - 1|','FontSize',FontSize);
title('kappa-test (cheap)','FontSize',FontSize);
set(gca,'XLim',[eps(1) eps(end)],'FontSize',FontSize-2); box off;

% Kappa-test #2: expensive test

epS = 10^-6; figure(3);
xlabel('control/gradient component #, i','FontSize',FontSize); ylabel('\kappa(i)','FontSize',FontSize); box off;
title('kappa-test (expensive)','FontSize',FontSize); set(gca,'FontSize',FontSize-2,'Xlim',[0 numCtrls]); hold on;

for i = 1:length(a)
  a_new = zeros(size(a)); a_new(i) = a(i);
  grad_new = zeros(size(grad_curr)); grad_new(i) = grad_curr(i);
  
  % dL2 inner product (in denominator) by gradient
  fid = fopen('temp/opt_grad.dat', 'w'); fprintf(fid, '%d\r\n', numCtrls);
  for k=1:numCtrls; fprintf(fid, '%1.20e\r\n', grad_new(k)); end; fclose(fid);
  paramsLine = ['-p1 ' num2str(numCtrls)];
  [status, cmdout] = system(['FreeFem++ freefem/kappa_grad.edp ' paramsLine]);
  fn_stat(status, 0, cmdout, ffDebug);
  G = load('temp/opt_kappa.dat');
  
  % computing kappa
  kappaE(i) = (f(epS,a,a_new,params)-obj)/(epS*G);
  
  % visualization for expensive kappa-test #2
  plot(i,kappaE(i),'r.','MarkerSize',MarkerSizeSmall); hold on; plot([0 numCtrls], [1 1], '--k'); hold on;

end