% mode_OPT.m
%
% OPTimization mode for Chapter_12_opt_pde.m
%

% main OPTimization loop for gradient-based methods
while(k < kMax+1) % termination condition #3

  % J-evaluation
  [objCurr aNormCurr] = f(0.0, a, zeros(size(a)), params);
  obj = [obj objCurr];
  aNorm = [aNorm aNormCurr];
  
  % output
  fprintf('iteration #%d: %1.8e\n',k,objCurr);

  % visualization
  visualize;

  % termination condition #1: checking optimality (objective change)
  if k > 0
        err1 = abs(obj(end-1)-obj(end))/abs(obj(end-1));
    if (err1 < epsilonJ)
      exCode = 1;
      break;
    end
  end

  % termination condition #2: checking optimality (solution change)
  if k > 0
    err2 = norm(a-a_old,2)/norm(a_old,2);
    if (err2 < epsilonA)
      exCode = 2;
      break;
    end
  end

  % choosing main solution method
  switch method
    
    % available methods
    case {'SD', 'CG', 'BFGS'}
      direction_search;         % d-search
      stepsize_search;          % alpha-search
      alpha = [alpha alphaOpt];
    
    otherwise
      disp(['error (OPT): Unknown method ' method ' is chosen!']);
      exCode = 0; % termination due to error
      return;

  end
  
  % update for controls
  a_old = a;
  a = a + alphaOpt*d;
  aHist = [aHist; a']; % for convergence analysis
  
  % iteration counter increment
  k = k + 1;

end

if (exCode==3)
  k = k - 1;
end

% convergence analysis
fn_convergence_sol_norm([],aHist);
