% visualize.m
%
% Visualization procedure for Chapter_12_opt_pde.m
%

% choosing color scheme
% (change CLRLAB variable to experiment with different schemes from colorlabels)
colorlabels = {'default','hsv','hot','pink','cool','bone','prism','flag','gray','rand'};
set(groot, 'DefaultFigureColormap', jet); CLRLAB = 1; colormap(colorlabels{CLRLAB});

% open big figure with 6 graphs (first portion)
figure(1);
if (BigFigure)
  set(gcf, 'Units', 'Normalized', 'OuterPosition', [0 0 1 1]);
end

% graph #1 plotting: objective function
subplot(2,3,1);
semilogy(0:k,obj,'b.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1); hold on;
title(['objective, J = ' num2str(obj(end))],'FontSize',FontSize);
xlabel('iterations, k','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;

% graphs ## 2&5 plotting: true & current optimal controls a(x,y)
nx = visDiscr; ny = nx; N = nx*ny; x = 1:nx; y = 1:ny;
fid = fopen('temp/opt_vis_a_true.dat','r'); temp = fscanf(fid,'%e',[1 N]);
for j = y; a_true(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);
fid = fopen('temp/opt_vis_a_curr.dat','r'); temp = fscanf(fid,'%e',[1 N]);
for j = y; a_curr(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);

subplot(2,3,2);
imagesc(x,y,a_true); axis xy; axis square; box off; colorbar;
caxis([min(min([a_true a_curr])) max(max([a_true a_curr]))]);
set(gca,'XTick',[]); set(gca,'YTick',[]);
title('control a(x,y): true','FontSize',FontSize); set(gca,'FontSize',FontSize); 

subplot(2,3,5);
imagesc(x,y,a_curr); axis xy; axis square; box off; colorbar;
caxis([min(min([a_true a_curr])) max(max([a_true a_curr]))]);
set(gca,'XTick',[]); set(gca,'YTick',[]);
title(['current a(x,y), error = ' num2str(aNorm(end))],'FontSize',FontSize); set(gca,'FontSize',FontSize); 

% graphs ## 3&6 plotting: true & current states u(x,y)
fid = fopen('temp/opt_vis_u_true.dat','r'); temp = fscanf(fid,'%e',[1 N]);
for j = y; u_true(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);
fid = fopen('temp/opt_vis_u_curr.dat','r'); temp = fscanf(fid,'%e',[1 N]);
for j = y; u_curr(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);

subplot(2,3,3);
imagesc(x,y,u_true); axis xy; axis square; box off; colorbar;
caxis([min(min([u_true u_curr])) max(max([u_true u_curr]))]);
set(gca,'XTick',[]); set(gca,'YTick',[]);
title('state u(x,y): true','FontSize',FontSize); set(gca,'FontSize',FontSize);

subplot(2,3,6);
imagesc(x,y,u_curr); axis xy; axis square; box off; colorbar;
caxis([min(min([u_true u_curr])) max(max([u_true u_curr]))]);
set(gca,'XTick',[]); set(gca,'YTick',[]);
title('state u(x,y): current','FontSize',FontSize); set(gca,'FontSize',FontSize);

% graph #4 plotting: gradient
if k>0
  subplot(2,3,4);
  fid = fopen('temp/opt_vis_grad_curr.dat','r'); temp = fscanf(fid,'%e',[1 N]);
  for j = y; gradV(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);
  imagesc(x,y,gradV); axis xy; axis square; box off; colorbar;
  set(gca,'XTick',[]); set(gca,'YTick',[]);
  title('gradient','FontSize',FontSize); set(gca,'FontSize',FontSize); 
end

% open big figure with rest 3 graphs (second portion)
figure(2);
if (BigFigure)
  set(gcf, 'Units', 'Normalized', 'OuterPosition', [0 0 1 1]);
end

% graph #1 plotting: L2-norm error in control
subplot(2,3,1);
semilogy(0:k,aNorm,'r.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1); hold on;
title(['L_2-norm error = ' num2str(aNorm(end))],'FontSize',FontSize);
xlabel('iterations, k','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;

% graph #2 plotting: step size tracking
if (k>0)
  subplot(2,3,2);
  semilogy(1:k,alpha,'b.','MarkerSize',MarkerSizeSmall); hold on;
  title(['step size, \alpha = ' num2str(alpha(end))],'FontSize',FontSize);
  xlabel('iterations, k','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;
end

% graph #3 plotting: convergence analysis (at the very end)

% graph #4 plotting: adjoint solution psi(x,y)
if k>0
  subplot(2,3,4);
  fid = fopen('temp/opt_vis_psi_curr.dat','r'); temp = fscanf(fid,'%e',[1 N]);
  for j = y; psi_curr(j,x) = temp((j-1)*nx+1:j*nx); end; fclose(fid);
  imagesc(x,y,psi_curr); axis xy; axis square; box off; colorbar;
  set(gca,'XTick',[]); set(gca,'YTick',[]);
  title('adjoint \psi(x,y)','FontSize',FontSize); set(gca,'FontSize',FontSize); 
end

% graphs ## 5-6 plotting: add your own ones

% make a pause
if pauseTime > 0
  pause(pauseTime);
end

% return to figure(1)
figure(1);