% fn_convergence_sol_norm.m
%
% To perform convergence analysis computationally
%

function fn_convergence_sol_norm(aEx,aHist)

  % choosing uOpt: use u(end) if not provided
  if isempty(aEx)
    aEx = aHist(end,:);
    aHist = aHist(1:end-1,:); % removing uEx value(s) from uOpt sequence
  end
  
  % checking if data size allows to complete analysis
  if size(aHist,1)<3
    disp('Convergence analysis cannot be completed due to the data size!');
    return;
  end

  % computational analysis using log10 and polyfit MATLAB functions
  for i = 1:size(aHist,1)
    ee(i) = norm(aHist(i,:)-aEx);
  end
  ee(ee == 0) = []; % removing zero values to avoid problems with log10
  x = log10(ee(1:end-1));
  y = log10(ee(2:end));
  d = polyfit(x,y,1);

  % visualizing results
  figure(2); subplot(2,3,3);
  xx = min(x):0.001:max(x);
  yy = polyval(d,xx);
  plot(xx,yy,'-r', 'LineWidth',2.5); hold on;
  plot(x,y,'b.','MarkerSize',20);
  xlabel('log_{10} |e^k|','FontSize',16);
  ylabel('log_{10} |e^{k+1}|','FontSize',16);
  title(['convergence: r = ' num2str(d(1)) ', C = ' num2str(10^d(2))]);
  set(gca,'FontSize',12); box off;

end