% Chapter_2_min_1D_Bisection.m
%
% Computational Optimization: Success in Practice
% Chapter 2: Minimization Approaches for Functions of One Variable
% Section 2.7: Practical Examples
%
% Coding in MATLAB: Bisection (Binary Search) Method
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; FontSize = 14;

% function f(x) and search interval [a, b]
fnNum = 1; % function number
xEx = []; % exact solution if known
a = -5; b = 5;
f = @(x) fn_all_functions(fnNum,x);

% method parameters
delta = 0.0001;
epsilon = 0.0002;

% check if parameters are correct
if (delta <= 0) || (delta >= b-a)
  disp('Incorrect delta!!!');
  return;
end
if (epsilon <= delta)
  disp('Incorrect epsilon!!!');
  return;
end

% plotting function f(x)
x = a:0.001:b;
figure(1); subplot(1,2,1);
plot(x,f(x),'b-','LineWidth',2.5); box off;
set(gca,'FontSize',FontSize); hold on;

% setting initial values for k, n, d
k = 0; % iteration count
n = 0; % number of function evaluations
d = b-a; % interval length (error)

% optimization loop
xOpt = (a+b)/2; % suboptimal solution
while(d(end) >= epsilon) % termination criterion
  if k > 0
    x1 = (a + b - delta)/2;
    x2 = (a + b + delta)/2;
    f1 = f(x1); f2 = f(x2);
    n = [n n(end)+2]; % 2 function evaluations
    if (f1 <= f2)
      b = x2;
    else
      a = x1;
    end
    d = [d b-a]; % updating interval length
    xOpt = [xOpt (a+b)/2]; % suboptimal solution
  end

  % visualization (current solution in red)
  subplot(1,2,1); plot([a b], [f(a) f(b)], 'r.', 'MarkerSize', 25); hold on;
  title(['k = ' num2str(k) ' iterations, d = ' num2str(d(end)) ', x* = ' num2str((a+b)/2)],'FontSize',FontSize); hold on;
  % visualization (interval length)
  subplot(1,2,2); semilogy(n,d,'b.-'); box off; hold on;
  xlabel(['function evaluations, n = 2k = ' num2str(n(end))],'FontSize',FontSize);
  title('interval length, d (log-scaled)','FontSize',FontSize);
  axis([0 40 1e-4 1e+1]); set(gca,'FontSize',FontSize);
  pause(1);

  k = k + 1;
   
  % visualization (current solution in blue to "disappear")
  subplot(1,2,1); plot([a b], [f(a) f(b)], 'b.', 'MarkerSize', 25); hold on;
end

% visualization (optimal solution at the end)
subplot(1,2,1); plot([a b], [f(a) f(b)], 'r.', 'MarkerSize', 25);
title(['k = ' num2str(k-1) ' iterations, d = ' num2str(d(end)) ...
    ' < \epsilon = ' num2str(epsilon) ', x* = ' num2str(xOpt(end))],'Color','r','FontSize',FontSize);

% convergence analysis
fn_convergence(xEx,xOpt);
