% Chapter_2_min_1D_BruteForce.m
%
% Computational Optimization: Success in Practice
% Chapter 2: Minimization Approaches for Functions of One Variable
% Section 2.7: Practical Examples
%
% Coding in MATLAB: Exhaustive (Brute-Force) Search Method
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; FontSize = 14;

% function f(x) and search interval [a, b]
fnNum = 1; % function number
a = -1; b = 3;
f = @(x) fn_all_functions(fnNum,x);

% method parameters
n = 3; % number of inner points
h = (b-a)/(n+1); % step size

% plotting function f(x)
x = a:0.001:b;
figure(1); plot(x,f(x),'b-','LineWidth',2.5); box off;
set(gca,'FontSize',FontSize); hold on;

% discretizing x
x = a:h:b;

% setting left boundary as a candidate solution (initial)
fn(1) = f(a);
x_bar = a;
f_bar = fn(1);

% visualization (initial)
plot(a, fn(1), 'r.', 'MarkerSize', 25);
title(['k = 1 iteration (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize); hold on;
pause(1);

% optimization loop
for k=2:n+2
  fn(k) = f(x(k)); % function evaluation
  % visualization (linear spline interpolation)
  plot([x(k-1) x(k)], [fn(k-1) fn(k)], 'm--','LineWidth',1.5); hold on;
  if fn(k) < f_bar
    plot(x_bar, f_bar, 'b.', 'MarkerSize', 25); hold on;
    x_bar = x(k);
    f_bar = fn(k);
    % visualization (optimal solution in red)
    plot(x_bar, f_bar, 'r.', 'MarkerSize', 25); hold on;
  else
    % visualization (non-optimal solutions in blue)
    plot(x(k), fn(k), 'b.', 'MarkerSize', 25); hold on;
  end
  title(['k = ' num2str(k) ' iterations (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize); hold on;
  pause(1);
end

% visualization (title for optimal solution at the end)
title(['k = ' num2str(k) ' iterations (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize,'Color','r'); hold off;
