% Chapter_2_min_1D_MonteCarlo.m
%
% Computational Optimization: Success in Practice
% Chapter 2: Minimization Approaches for Functions of One Variable
% Section 2.7: Practical Examples
%
% Coding in MATLAB: Monte Carlo (Random Search) Method
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; FontSize = 14;

% function f(x) and search interval [a, b]
fnNum = 1; % function number
a = -5; b = 5;
f = @(x) fn_all_functions(fnNum,x);

% method parameter
n = 10; % number of samples

% plotting function f(x)
x = a:0.001:b;
figure(1); plot(x,f(x),'b-','LineWidth',2.5); box off;
set(gca,'FontSize',FontSize); hold on;

% compution first (initial) x and associated function
% using MATLAB command rand
% which returns a single uniformly distributed random number in the interval [0,1]
x_rnd = a + (b-a)*rand; % random number in the interval [a,b]
x_bar = x_rnd;
f_bar = f(x_rnd);

% visualization (initial)
plot(x_bar, f_bar, 'r.', 'MarkerSize', 25);
title(['k = 1 iteration (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize); hold on;
pause(2);

% optimization loop
for k=2:n
  x_rnd = a + (b-a)*rand; % new random value of x
  f_rnd = f(x_rnd);       % function evaluation
  if f_rnd < f_bar
    plot(x_bar, f_bar, 'b.', 'MarkerSize', 25); hold on;
    x_bar = x_rnd;
    f_bar = f_rnd;
    % visualization (optimal solution in red)
    plot(x_bar, f_bar, 'r.', 'MarkerSize', 25); hold on;
  else
    % visualization (non-optimal solutions in blue)
    plot(x_rnd, f_rnd, 'b.', 'MarkerSize', 25); hold on;
  end
  title(['k = ' num2str(k) ' iterations (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize); hold on;
  pause(1);
end

% visualization (title for optimal solution at the end)
title(['k = ' num2str(k) ' iterations (f evaluations), x* = ' num2str(x_bar)],'FontSize',FontSize,'Color','r'); hold off;
