% Chapter_2_min_1D_Newton.m
%
% Computational Optimization: Success in Practice
% Chapter 2: Minimization Approaches for Functions of One Variable
% Section 2.7: Practical Examples
%
% Coding in MATLAB: Newton's (2nd-order) Method
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; FontSize = 14;

% function f(x) and derivatives f'(x) and f''(x)
fnNum = 3; % function number
xEx = []; % exact solution if known
f  = @(x) fn_all_functions(fnNum,x);
fd = @(x) fn_all_derivatives(fnNum,x);

% interval [a, b]: for visualization
a = -2.5; b = 3.5;

% initial guess
xOpt = 1;

% method parameter
epsilon = 0.00001;

% plotting function f(x)
xx = a:0.001:b;
figure(1); plot(xx,f(xx),'b-','LineWidth',2.5); box off;
set(gca,'FontSize',FontSize); hold on;

% setting initial values for k, d
k = 0; % iteration count
n = 0; % number of function evaluations

% optimization loop
while(1)
  
  % two derivatives evaluations
  deriv = fd(xOpt(end));
  
  % checking if f'(x) does not exist
  if isnan(deriv(1))
    disp('Attention! 1st derivative does not exist: terminated!');
    break;
  end
  
  % checking if f''(x) = 0 (or very close to 0) or does not exist
  if abs(deriv(2))<1e-10 || isnan(deriv(2))
    disp('Attention! 2nd derivative is 0 or does not exist: terminated!');
    break;
  end
  
  % termination criterion
  if (abs(deriv(1)) < epsilon)
    break;
  end

  % solution update
  xOpt = [xOpt xOpt(end) - deriv(1)/deriv(2)]; % solution update
  k = k + 1;

  % visualization (old solution in blue to "disappear")
  plot(xOpt(end-1), f(xOpt(end-1)), 'b.', 'MarkerSize', 25); hold on;
  % visualization (current solution in red)
  plot(xOpt(end), f(xOpt(end)), 'r.', 'MarkerSize', 25); hold on;
  title(['k = ' num2str(k) ' iterations, f'' = ' num2str(deriv(1)) ', x* = ' num2str(xOpt(end))],'FontSize',FontSize); hold on;
  pause(1);
  
end

% visualization (optimal solution at the end)
title(['k = ' num2str(k) ' iterations, f'' = ' num2str(abs(deriv(1))) ...
    ' < \epsilon = ' num2str(epsilon) ', x* = ' num2str(xOpt(end))],'Color','r','FontSize',FontSize); hold off;

% convergence analysis
fn_convergence(xEx,xOpt);
