% fn_convergence.m
%
% To perform convergence analysis computationally
%

function fn_convergence(xEx,xOpt)

  % choosing xOpt: use x(end) if not provided
  if isempty(xEx)
    xEx = xOpt(end);
    xOpt(xOpt == xEx) = []; % removing xEx value(s) from xOpt sequence
  end
  
  % checking if data size allows to complete analysis
  if length(xOpt)<3
    disp('Convergence analysis cannot be completed due to the data size!');
    return;
  end

  % computational analysis using log10 and polyfit MATLAB functions
  x = log10(abs(xOpt(1:end-1)-xEx));
  y = log10(abs(xOpt(2:end)-xEx));
  d = polyfit(x,y,1);

  % visualizing results
  figure;
  xx = min(x):0.001:max(x);
  yy = polyval(d,xx);
  plot(xx,yy,'-r', 'LineWidth',2.5); hold on;
  plot(x,y,'b.','MarkerSize',20);
  xlabel('log_{10} |e^k|','FontSize',16);
  ylabel('log_{10} |e^{k+1}|','FontSize',16);
  title(['convergence: r = ' num2str(d(1)) ', C = ' num2str(10^d(2))]);
  set(gca,'FontSize',16); box off;

end