% Chapter_3_data_fit_by_gradient_ver_3.m
%
% Computational Optimization: Success in Practice
% Chapter 3: Generalized Optimization Framework
% Section 3.7: Accuracy and Performance
%
% Coding in MATLAB: Optimization Framework for Example 1.3
% Least-square Data Fitting by Gradient Method
% ver. 3 improvement: applying 2-order Newton's method for search direction
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear;

% setting INPUT parameters
params_ver_3;

% loading DATA
data = load(dataFile);

% INItialization
initialize_ver_3;

% main OPTimization loop
while(k < kMax+1) % termination condition #3

  % f-evaluation
  obj = [obj f(a, data)];

  % visualization
  visualize;

  % termination condition #1: checking optimality (objective change)
  if k > 0
    err1 = abs(obj(end-1)-obj(end))/obj(end-1);
    if (err1 < epsilonF)
      break;
    end
  end

  % termination condition #2: checking optimality (solution change)
  if k > 0
    err2 = norm(a-a_old,2)/norm(a_old,2);
    if (err2 < epsilonA)
      break;
    end
  end
    
  % search for d: computing using Newton's method
  d = -inv(hess(a,data))*grad(a,data);
   
  % search for alpha (golden section vs. constant)
  if (alphaUpdate)
    golden_section_search;
  else
    alphaOpt = 1;
  end
  alpha = [alpha alphaOpt];
  
  % update for controls
  a_old = a;
  a = a + alpha(end)*d;
  aHist = [aHist a];
   
  % iteration counter increment
  k = k + 1;

end

% convergence analysis
fn_convergence_sol_norm([],aHist);
