% initialize.m
%
% Initialization procedure for Chapter_6_data_fit_by_line_search.m
%

% initial guess for controls
a = aini';

% saving shape of curve at initial guess
x = min(data(:,1))-1:0.01:max(data(:,1)+1);
yini = a(1)+a(2)*x+a(3)*x.^2;

% initializing variables and vectors
k = 0;      % number of iterations
obj = [];   % objective function evaluations
aHist = a;  % history of control changes
alpha = []; % stepsize records
exCode = 3; % OPT termination condition (#1, #2 or #3)

% initializing golden section search parameters (if GS is chosen)
if strcmp(methodAlpha,'GS')
  alphaGS = (-1+sqrt(5))/2;
  if (alphaEps <= 0) || (alphaEps >alphaB-alphaA)
    disp('Incorrect epsilon!!!');
    return;
  end
end

% function handlers
f           = @fn_eval_f;           % for objective function f(x)
grad_f      = @fn_eval_f_grad;      % for gradient of f(x)
hess        = @fn_eval_hess;        % for Hessian of f(x)
hess_approx = @fn_eval_hess_approx; % for FD-approximated Hessian of f(x)
g           = @fn_eval_g;           % for Gauss_Newton g(x)
grad_g      = @fn_eval_g_grad;      % for Gauss_Newton gradient of g(x)
