% stepsize_search.m
%
% Algorithm for finding optimal stepsize in line search by:
% - constant (CONST)
% - Golden Section (GS)
% - diminishing (DIM)
% - backtracking (BLS, BLSW, BLSSW, BLSG)
% - bracketing-Brent toolbox (BB)
%

switch methodAlpha
    
  case 'CONST'
    alphaOpt = alphaConst;
    
  case 'GS'
    golden_section_search;
    
  case 'DIM'
    if (k==0)
      alphaOpt = alphaIni;
    else
      alphaOpt = alphaOpt * alphaRho;
    end
    
  case 'BLS'
    alphaOpt = alphaIni;
    % sufficient decrease condition
    while(f(a+alphaOpt*d,data) > f(a,data)+alphaC*alphaOpt*(grad_f(a,data))'*d)
      alphaOpt = alphaOpt*alphaRho;
    end
    
  case 'BLSW'
    alphaOpt = alphaIni;
    % Wolfe conditions
    while(f(a+alphaOpt*d,data) > f(a,data)+alphaC*alphaOpt*(grad_f(a,data))'*d ...
        || (grad_f(a+alphaOpt*d,data))'*d < alphaCw*(grad_f(a,data))'*d)
      alphaOpt = alphaOpt*alphaRho;
    end
    
  case 'BLSSW'
    alphaOpt = alphaIni;
    % strong Wolfe conditions
    while(f(a+alphaOpt*d,data) > f(a,data)+alphaC*alphaOpt*(grad_f(a,data))'*d ...
        || abs((grad_f(a+alphaOpt*d,data))'*d) < abs(alphaCw*(grad_f(a,data))'*d))
      alphaOpt = alphaOpt*alphaRho;
    end

  case 'BLSG'
    alphaOpt = alphaIni;
    % Goldstein conditions
    while(f(a+alphaOpt*d,data) > f(a,data)+alphaC*alphaOpt*(grad_f(a,data))'*d ...
        || f(a+alphaOpt*d,data) < f(a,data)+(1-alphaC)*alphaOpt*(grad_f(a,data))'*d)
      alphaOpt = alphaOpt*alphaRho;
    end

  case 'BB'
    % initial bracketing guess
    AX = AXini; BX = BXini;
    FA = FUNC(AX,a,d,f,data); FB = FUNC(BX,a,d,f,data);
    % calling bracketing routine
    [AX,FA,BX,FB,CX,FC] = fn_min_brack(AX,BX,FA,FB,@(alphaCurr) FUNC(alphaCurr,a,d,f,data),MAXITER,GLIMIT);
    % calling minimization routine
    [BX,FB] = fn_brent(AX,BX,CX,FA,FB,FC,@(alphaCurr) FUNC(alphaCurr,a,d,f,data),TOL,ITMAX);
    alphaOpt = BX;
    % using BX from previous iterations
    BXini = alphaOpt;
    
  otherwise
    disp(['error (OPT): Unknown stepsize search method ' methodAlpha ' is chosen!']);
    exCode = 0; % termination due to error
    return;

end
