% visualize.m
%
% Visualization procedure for
% Chapter_5_data_fit_by_line_search.m
% and
% Chapter_6_data_fit_by_line_search.m
%


% open big figure with 4 graphs
figure(1);
if (BigFigure)
  set(gcf, 'Units', 'Normalized', 'OuterPosition', [0 0 1 1]);
end

% graph #1 plotting
% data points
subplot(2,2,1);
plot(data(:,1), data(:,2), 'r.', 'MarkerSize', MarkerSizeBig); hold on;

% initial guess
plot(x, yini, 'b--', 'LineWidth', LineWidth); hold on;
   
% fitting curve
y = a(1)+a(2)*x+a(3)*x.^2;
plot(x, y, 'b-', 'LineWidth', LineWidth); hold off;

% graph #1 settings
title(['k = ' num2str(k) ', y = ' num2str(a(1)) ' + ' num2str(a(2)) ' * x + ' num2str(a(3)) ' * x^2'],'FontSize',FontSize);
set(gca,'FontSize',FontSize,'xLim',[min(x) max(x)]); box off;

% graph #2 plotting
% objective function
subplot(2,2,2);
semilogy(0:k,obj,'b.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1); hold on;

% graph #2 settings
xlabel('iterations, k','FontSize',FontSize);
title(['objective function (log-scaled), f = ' num2str(obj(end))],'FontSize',FontSize);
set(gca,'FontSize',FontSize); box off;

% graph #3 plotting
% search direction
if k>0
  subplot(2,2,3);
  plot([1 length(d)],[0 0],'k--','LineWidth',LineWidth-1);hold on;
  for i = 1:length(d)
    plot([i i],[0 d(i)],'r.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth); hold on;
    plot(i,d(i),'rs','MarkerFaceColor','r','MarkerSize',MarkerSizeSmall/2); hold on;
  end
  hold off;
  % graph #3 settings
  xlabel('components','FontSize',FontSize);
  title(['search direction d, norm = ' num2str(norm(d,2))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
end

% graph #4 plotting
% stepsize change tracking
if k>0
  subplot(2,2,4);
  semilogy(1:k,alpha,'b.','MarkerSize',MarkerSizeSmall); hold on;
  % graph #4 settings
  xlabel('iterations, k','FontSize',FontSize);
  title(['step size (log-scaled), \alpha = ' num2str(alpha(end))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
end

% make a pause
if pauseTime > 0
  pause(pauseTime);
end
