% Chapter_7_data_fit_comprehensive.m
%
% Computational Optimization: Success in Practice
% Chapter 7: Trust Region Methods
% Section 7.5: Checking and Tuning Performance
% Section 7.6: Exploring Derivative-Free Options
%
% Coding in MATLAB: Optimization Framework for Example 1.3
% Least-square Data Fitting by Different Gradient-based Methods
% Included:
% - modes: regular OPT & TEST (for gradient-based approaches)
% - methods (gradient-based):
%          * steepest descent (SD)
%          * pure Newton's method (NEWTON)
%          * discretized Newton's (DN)
%          * modified Newton's (MN)
%          * diagonally scaled SD (DSSD)
%          * quasi-Newton (BFGS)
%          * Gauss--Newton (GN)
%          * Conjugate Gradient (CG)
%          * trust region dogleg (TRDL)
% - methods (non-gradient):
%          * brute-force (BF)
%          * stochastic Monte-Carlo (MC)
% - stepsize methods:
%          * constant (CONST)
%          * Golden Section (GS) Search
%          * diminishing (DIM)
%          * backtracking (BLS)
%          * BLS with Wolfe conditions (BLSW)
%          * BLS with strong Wolfe conditions (BLSSW)
%          * BLS with Goldstein conditions (BLSG)
%          * bracketing-Brent toolbox (BB)
% - output: functional value (in file)
%           iterative solutions (in file)
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear; tic;

% setting INPUT parameters
params;

% paths for supplemental MATLAB codes
addpath algorithms; % algorithms for main code
addpath functions;  % m-functions

% loading DATA
data = load(dataFile);

% INItialization
initialize;

% choosing mode
if strcmp(mode,'OPT')
  mode_OPT;
elseif strcmp(mode,'TEST')
  mode_TEST;
else
  disp(['error: Unknown mode ' mode ' is chosen!']); return;
end

% final output
if (exCode==0)
  return;
elseif strcmp(mode,'OPT')
  file_output;
  fprintf(['Terminated in ' num2str(k) ' iteration(s): termination condition #' num2str(exCode) ' satisfied!\n']);
end
fprintf(['We are fully done! CPU elapsed time = ' num2str(toc) 's\n\n']);
