% mode_OPT.m
%
% OPTimization mode for Chapter_7_data_fit_comprehensive.m
%

% OPTimization by gradient-based and derivative-free methods
switch method
  
  case 'BF' % brute-force
    aAll = []; % to supress output in file at the end
    brute_force_method;

  case 'MC' % Monte-Carlo
    aAll = []; % to supress output in file at the end
    monte_carlo_method;

  otherwise % gradient-based methods

    while(k < kMax+1) % termination condition #3

      % f-evaluation
      obj = [obj f(a, data)];

      % visualization
      visualize;

      % termination condition #1: checking optimality (objective change)
      if k > 0
        err1 = abs(obj(end-1)-obj(end))/abs(obj(end-1));
        if (err1 < epsilonF)
          exCode = 1;
          break;
        end
      end

      % termination condition #2: checking optimality (solution change)
      if k > 0
        err2 = norm(a-a_old,2)/norm(a_old,2);
        if (err2 < epsilonA)
          exCode = 2;
          break;
        end
      end
    
      % choosing main solution method
      switch method
   
        % Line Search methods
        case {'SD', 'NEWTON', 'DN', 'MN', 'DSSD', 'BFGS', 'GN', 'CG'}
          direction_search;         % d-search
          stepsize_search;          % alpha-search
          p = alphaOpt*d;           % solution update
          alpha = [alpha alphaOpt];
  
        % Trust Region methods
        case 'TRDL'
          trust_region_method;

        otherwise
          disp(['error (OPT): Unknown method ' method ' is chosen!']);
          exCode = 0; % termination due to error
          return;

      end
  
      % update for controls
      a_old = a;
      a = a + p;
      aHist = [aHist a]; % for convergence analysis
   
      % iteration counter increment
      k = k + 1;

    end

  if (exCode==3)
    k = k - 1;
  end

end

% convergence analysis
fn_convergence_sol_norm([],aHist);
