% trust_region_method.m
%
% Algorithm for finding trust region candidate step using:
% - Dogleg method (TRDL)
%

switch method
  
  % Dogleg method
  case 'TRDL'
    delta = deltaTR(end);
    
    % search for p
    grad = grad_f(a,data); % one time gradient computation for multiple use
    B = hess(a,data);      % one time Hessian computation for multiple use
    pB = -inv(B)*grad;
    if (norm(pB,2)<delta)
      p = pB;
    else
      pU = -((grad'*grad)/(grad'*B*grad))*grad;
      % search for tau
      if (norm(pU,2)>=delta)
        tau = delta/norm(pU,2);
      else
        % solving scalar quadratic equation
        TRa = (pB-pU)'*(pB-pU);
        TRb = 2 * pU' * (pB-pU);
        TRc = pU'*pU - delta^2;
        tau = (-TRb + sqrt(TRb^2-4*TRa*TRc))/(2*TRa)+1;
      end
      % computing p
      if (tau <= 1)
        p = tau*pU;
      else
        p = pU + (tau-1)*(pB-pU);
      end
    end

    % update for delta
    rho = (f(a,data)-f(a+p,data))/(-p'*grad+0.5*p'*B*p);
    if (rho<0.25)
      delta = 0.25*norm(p,2);
    elseif ( (rho>0.75) && (norm(p,2)==delta) )
      delta = min(2*delta,deltaTRmax);
    end
    deltaTR = [deltaTR delta];
    
    if (rho<=paramTR)
      p = zeros(size(p));
    end
    
  otherwise
    disp(['error (OPT): Unknown trust region method ' method ' is chosen!']);
    exCode = 0; % termination due to error
    return;

end
