% Chapter_8_opt_constrained.m
%
% Computational Optimization: Success in Practice
% Chapter 8: Large-Scale and Constrained Optimization
% Section 8.5: Lagrange Multiplier Approach
%
% Coding in MATLAB: Optimization Framework for Example 8.1
% Quadratic Objective + Linear Equality Constraint
% Solution by Lagrange multiplier approach
% Included:
% - modes: regular OPT & TEST (for gradient-based approaches)
% - methods (gradient-based):
%          * steepest descent (SD)
%          * pure Newton's method (NEWTON)
%          * discretized Newton's (DN)     <- deactivated
%          * modified Newton's (MN)        <- deactivated
%          * diagonally scaled SD (DSSD)   <- deactivated
%          * quasi-Newton (BFGS)
%          * Gauss--Newton (GN)            <- deactivated
%          * Conjugate Gradient (CG)
%          * trust region dogleg (TRDL)    <- deactivated
% - methods (non-gradient):
%          * brute-force (BF)              <- deactivated
%          * stochastic Monte-Carlo (MC)   <- deactivated
% - stepsize methods:
%          * constant (CONST)
%          * Golden Section (GS) Search
%          * diminishing (DIM)
%          * backtracking (BLS)
%          * BLS with Wolfe conditions (BLSW)
%          * BLS with strong Wolfe conditions (BLSSW)
%          * BLS with Goldstein conditions (BLSG)
%          * bracketing-Brent toolbox (BB)
% - output: functional value (in file)
%           iterative solutions (in file)
%
%  2023 by Vladislav Bukshtynov
% VladislavBukshtynov@yahoo.com
%

close all; clc; clear; tic;

% setting INPUT parameters
params;

% paths for supplemental MATLAB codes
addpath algorithms; % algorithms for main code
addpath functions;  % m-functions

% INItialization
initialize;

% choosing mode
if strcmp(mode,'OPT')
  mode_OPT;
elseif strcmp(mode,'TEST')
  mode_TEST;
else
  disp(['error: Unknown mode ' mode ' is chosen!']); return;
end

% final output
if (exCode==0)
  return;
elseif strcmp(mode,'OPT')
  file_output;
  fprintf(['Terminated in ' num2str(k) ' iteration(s): termination condition #' num2str(exCode) ' satisfied!\n']);
end
fprintf(['We are fully done! CPU elapsed time = ' num2str(toc) 's\n\n']);
