% direction_search.m
%
% Algorithm for finding search direction using:
% - Steepest Descent (SD)
% - Newton's method (NEWTON)
% - discretized Newton's (DN)
% - modified Newton's (MN)
% - diagonally scaled SD (DSSD)
% - quasi-Newton (BFGS)
% - Gauss--Newton (GN)
% - Conjugate Gradient (CG)
%

switch method
    
  case 'SD'
    d = -grad_f(a,data);
    
  case 'NEWTON'
    size(hess(a,data))
    size(grad_f(a,data))
    d = -inv(hess(a,data))*grad_f(a,data);
    
  case 'DN'
    d = -inv(hess_approx(a,data,stepH,f))*grad_f(a,data);
    
  case 'MN'
    if (k==0) 
      d = -inv(hess_approx(a,data,stepH,f))*grad_f(a,data);
    end
    
  case 'DSSD' % both ways are possible
    d = -(diag(1./diag(hess_approx(a,data,stepH,f))))*grad_f(a,data);
    %d = -inv(diag(diag(hess_approx(a,data,da,f))))*grad_f(a,data);
    
  case 'BFGS'
    if (k==0)
      D = eye(length(a),length(a));
      grad_old = grad_f(a,data);
    else
      grad_curr = grad_f(a,data);
      s = a - a_old;
      y = grad_curr - grad_old;
      grad_old = grad_curr;
      r = 1/(y'*s);
      D = (eye(length(a),length(a))-r*s*y')*D*(eye(length(a),length(a))-r*y*s')+r*s*s';
    end
    d = -D*grad_f(a,data);
    
  case 'GN'
    d = -inv((grad_g(a,data))'*grad_g(a,data))*(grad_g(a,data))'*g(a,data);
    
  case 'CG'
    grad_curr = grad_f(a,data);
    if (k==0)
      d = -grad_curr;
    else
      d = -grad_curr + (grad_curr'*grad_curr)/(grad_old'*grad_old)*d;
    end
    grad_old = grad_curr;
    
  otherwise
    disp(['error (OPT): Unknown search direction method ' method ' is chosen!']);
    exCode = 0; % termination due to error
    return;

end
