% initialize.m
%
% Initialization procedure for Chapter_8_opt_constrained.m
%

% initial guess for controls
a = aini';

% using data vector for passing some info to functions
data = [exMethod penalCoeff];

% true solution
aTrue = [1 1 1 1];

% initializing variables and vectors
k = 0;        % number of iterations
obj = [];     % objective function evaluations
aHist = a;    % history of control changes
alpha = [];   % stepsize records
deltaTR = []; % trust region size records
exCode = 3;   % OPT termination condition (#1, #2 or #3)

% initializing trust region method parameters (if TRDL is chosen)
if strcmp(method,'TRDL')
  methodAlpha = '';
  deltaTR = deltaTRmax*0.5; % intitial size of trust region
end

% initializing non-gradient methods parameters (if BF or MC is chosen)
if strcmp(method,'BF') || strcmp(method,'MC')
  methodAlpha = '';
end

% initializing golden section search parameters (if GS is chosen)
if strcmp(methodAlpha,'GS')
  alphaGS = (-1+sqrt(5))/2;
  if (alphaEps <= 0) || (alphaEps >alphaB-alphaA)
    disp('Incorrect epsilon!!!');
    return;
  end
end

% function handlers
f           = @fn_eval_f;           % for objective function f(x)
grad_f      = @fn_eval_f_grad;      % for gradient of f(x)
hess        = @fn_eval_hess;        % for Hessian of f(x)
hess_approx = @fn_eval_hess_approx; % for FD-approximated Hessian of f(x)
g           = @fn_eval_g;           % for Gauss_Newton g(x)
grad_g      = @fn_eval_g_grad;      % for Gauss_Newton gradient of g(x)
