% visualize.m
%
% Visualization procedure for Chapter_8_opt_constrained.m
%

% open big figure with 4 graphs
figure(1);
if (BigFigure)
  set(gcf, 'Units', 'Normalized', 'OuterPosition', [0 0 1 1]);
end

% graph #1 plotting
subplot(2,2,1);

% true solution
plot(aTrue, 'ro', 'MarkerSize', MarkerSizeSmall-5,'LineWidth',LineWidth-1); hold on;

% initial guess
if (~strcmp(method,'BF')) && ~(strcmp(method,'MC'))
  plot(aHist(:,1), 'bo', 'MarkerSize', MarkerSizeSmall-10,'LineWidth',LineWidth-1); hold on;
end
   
% current solution
aCurr = aHist(:,end);
if (~data(1))
  aCurr(4) = 4-sum(aCurr(1:3));
end
plot(aCurr, 'b.', 'MarkerSize', MarkerSizeBig); hold off;

% graph #1 settings
title(['k = ' num2str(k) ', x = [' num2str(aCurr') ']'],'FontSize',FontSize);
set(gca,'FontSize',FontSize); box off;

% graph #2 plotting
% objective function
subplot(2,2,2);
if (strcmp(method,'BF')) || (strcmp(method,'MC'))
  plot(1:length(obj),obj,'b.','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1); hold on;
  xlabel('solution updates, n','FontSize',FontSize);
else
  plot(0:k,obj,'b.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1); hold on;
  xlabel('iterations, k','FontSize',FontSize);
end

% graph #2 common settings
title(['objective, f = ' num2str(obj(end)) ' [res = ' num2str(4-sum(aCurr)) ']'],'FontSize',FontSize);
set(gca,'FontSize',FontSize); box off;

% graph #3 plotting
if ( (k>0) && (strcmp(method,'TRDL')) ) % trust-region step direction
  subplot(2,2,3);
  plot([1 length(p)],[0 0],'k--','LineWidth',LineWidth-1);hold on;
  for i = 1:length(p)
    plot([i i],[0 p(i)],'r.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth); hold on;
    plot(i,p(i),'rs','MarkerFaceColor','r','MarkerSize',MarkerSizeSmall/2); hold on;
  end
  hold off;
  % graph #3 settings
  xlabel('components','FontSize',FontSize);
  title(['trust-region step p, norm = ' num2str(norm(p,2))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
elseif ( (k>0) && (~isempty(methodAlpha)) ) % line search direction
  subplot(2,2,3);
  plot([1 length(d)],[0 0],'k--','LineWidth',LineWidth-1);hold on;
  for i = 1:length(d)
    plot([i i],[0 d(i)],'r.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth); hold on;
    plot(i,d(i),'rs','MarkerFaceColor','r','MarkerSize',MarkerSizeSmall/2); hold on;
  end
  hold off;
  % graph #3 settings
  xlabel('components','FontSize',FontSize);
  title(['search direction d, norm = ' num2str(norm(d,2))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
end

% graph #4 plotting
% step size tracking
if ( (k>0) && (~isempty(methodAlpha)) )
  subplot(2,2,4);
  semilogy(1:k,alpha,'b.','MarkerSize',MarkerSizeSmall); hold on;
  % graph #4 settings
  xlabel('iterations, k','FontSize',FontSize);
  title(['step size (log-scaled), \alpha = ' num2str(alpha(end))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
end
% trust-region size tracking
if (strcmp(method,'TRDL'))
  subplot(2,2,4);
  semilogy(0:k,deltaTR,'b.','MarkerSize',MarkerSizeSmall); hold on;
  % graph #4 settings
  xlabel('iterations, k','FontSize',FontSize);
  title(['trust-region radius (log-scaled), \Delta =' num2str(deltaTR(end))],'FontSize',FontSize);
  set(gca,'FontSize',FontSize); box off;
end

% make a pause
if pauseTime > 0
  pause(pauseTime);
end
