% stepsize_search.m
%
% Algorithm for finding optimal stepsize in line search by:
% - constant (CONST)
% - Golden Section (GS)
% - bracketing-Brent toolbox (BB)
%

switch methodAlpha
    
  case 'CONST'
    alphaOpt = alphaConst;
    
  case 'GS'
    golden_section_search;
    
  case 'BB'
    % initial bracketing guess
    AX = AXini; BX = BXini;
    FA = FUNC(AX,u,d,f,params,data); FB = FUNC(BX,u,d,f,params,data);
    % calling bracketing routine
    [AX,FA,BX,FB,CX,FC] = fn_min_brack(AX,BX,FA,FB,@(alphaCurr) FUNC(alphaCurr,u,d,f,params,data),MAXITER,GLIMIT);
    % calling minimization routine
    [BX,FB] = fn_brent(AX,BX,CX,FA,FB,FC,@(alphaCurr) FUNC(alphaCurr,u,d,f,params,data),TOL,ITMAX);
    alphaOpt = BX;
    % using BX from previous iterations
    BXini = alphaOpt;
    
  otherwise
    disp(['error (OPT): Unknown stepsize search method ' methodAlpha ' is chosen!']);
    exCode = 0; % termination due to error
    return;

end
