% visualize.m
%
% Visualization procedure for Chapter_9_opt_lotka_volterra.m
%

% open big figure with 6 graphs
figure(1);
if (BigFigure)
  set(gcf, 'Units', 'Normalized', 'OuterPosition', [0 0 1 1]);
end

% graph #1 plotting: objective function
subplot(2,3,1);
semilogy(0:k,obj,'b.-','MarkerSize',MarkerSizeSmall,'LineWidth',LineWidth-1);
title(['objective function, f = ' num2str(obj(end))],'FontSize',FontSize);
xlabel('iterations, k','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;

% graph #2 plotting: optimal control u(t)
subplot(2,3,2);
plot(tt,u_ini,'--k', tt,u,'o-b', tt,u_ex,'.-r');
title(['control u(t): iteration #' num2str(k)],'FontSize',FontSize);
legend('u_{ini}(t)','u(t)','u_{ex}(t)','Location','Best'); legend boxoff;
xlabel('t','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;

% graph #3 plotting: step size tracking
if (k>0)
  subplot(2,3,3);
  semilogy(1:k,alpha,'b.','MarkerSize',MarkerSizeSmall); hold on;
  title(['step size, \alpha = ' num2str(alpha(end))],'FontSize',FontSize);
  xlabel('iterations, k','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;
end

% graph #4 plotting: solution x_1(t) & x_2(t)
subplot(2,3,4);
plot(tt,data(:,2),'.-b', tt,data(:,3),'.-r', tt,x1,'o-b', tt,x2,'o-r', tt,x10,'--b', tt,x20,'--r');
title('solution: x_1(t) & x_2(t)','FontSize',FontSize);
legend('x_1(t)','x_2(t)','Location','Best'); legend boxoff;
xlabel('t','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;

% graph #5 plotting: gradient
if k>0
  subplot(2,3,5);
  plot(tt,-d,'.-b','MarkerSize',MarkerSizeSmall); hold on;
  plot(tt(end),-d(end),'.-r','MarkerSize',MarkerSizeBig); hold off;
  title('gradient','FontSize',FontSize);
  xlabel('t','FontSize',FontSize); set(gca,'FontSize',FontSize); box off;
end

% graph #6 plotting: convergence analysis (at the very end)

% make a pause
if pauseTime > 0
  pause(pauseTime);
end
